@@ -19,12 +19,12 @@ from pandas.core.common import (isnull, notnull, _values_from_object,
                                 ensure_float, _ensure_float64,
                                 _ensure_int64, _ensure_object,
                                 is_float, is_integer, is_complex,
-                                is_float_dtype, _is_floating_dtype,
+                                is_float_dtype, is_floating_dtype,
                                 is_complex_dtype, is_integer_dtype,
                                 is_bool_dtype, is_object_dtype,
                                 is_datetime64_dtype, is_timedelta64_dtype,
-                                _is_datetime_or_timedelta_dtype,
-                                _is_int_or_datetime_dtype, _is_any_int_dtype)
+                                is_datetime_or_timedelta_dtype,
+                                is_int_or_datetime_dtype, is_any_int_dtype)
 
 
 class disallow(object):
@@ -72,6 +72,10 @@ class bottleneck_switch(object):
             try:
                 if self.zero_value is not None and values.size == 0:
                     if values.ndim == 1:
+
+                        # wrap the 0's if needed
+                        if is_timedelta64_dtype(values):
+                            return lib.Timedelta(0)
                         return 0
                     else:
                         result_shape = (values.shape[:axis] +
@@ -101,7 +105,7 @@ class bottleneck_switch(object):
 def _bn_ok_dtype(dt, name):
     # Bottleneck chokes on datetime64
     if (not is_object_dtype(dt) and
-            not _is_datetime_or_timedelta_dtype(dt)):
+            not is_datetime_or_timedelta_dtype(dt)):
 
         # bottleneck does not properly upcast during the sum
         # so can overflow
@@ -194,7 +198,7 @@ def _get_values(values, skipna, fill_value=None, fill_value_typ=None,
 
 
 def _isfinite(values):
-    if _is_datetime_or_timedelta_dtype(values):
+    if is_datetime_or_timedelta_dtype(values):
         return isnull(values)
     if (is_complex_dtype(values) or is_float_dtype(values) or
             is_integer_dtype(values) or is_bool_dtype(values)):
@@ -203,11 +207,11 @@ def _isfinite(values):
 
 
 def _na_ok_dtype(dtype):
-    return not _is_int_or_datetime_dtype(dtype)
+    return not is_int_or_datetime_dtype(dtype)
 
 
 def _view_if_needed(values):
-    if _is_datetime_or_timedelta_dtype(values):
+    if is_datetime_or_timedelta_dtype(values):
         return values.view(np.int64)
     return values
 
@@ -222,19 +226,9 @@ def _wrap_results(result, dtype):
             result = result.view(dtype)
     elif is_timedelta64_dtype(dtype):
         if not isinstance(result, np.ndarray):
-
-            # this is a scalar timedelta result!
-            # we have series convert then take the element (scalar)
-            # as series will do the right thing in py3 (and deal with numpy
-            # 1.6.2 bug in that it results dtype of timedelta64[us]
-            from pandas import Series
-
-            # coerce float to results
-            if is_float(result):
-                result = int(result)
-            result = Series([result], dtype='timedelta64[ns]')
+            result = lib.Timedelta(result)
         else:
-            result = result.view(dtype)
+            result = result.astype('i8').view(dtype)
 
     return result
 
@@ -291,6 +285,7 @@ def nanmedian(values, axis=None, skipna=True):
 
     if values.dtype != np.float64:
         values = values.astype('f8')
+        values[mask] = np.nan
 
     if axis is None:
         values = values.ravel()
@@ -301,7 +296,7 @@ def nanmedian(values, axis=None, skipna=True):
     if values.ndim > 1:
         # there's a non-empty array to apply over otherwise numpy raises
         if notempty:
-            return np.apply_along_axis(get_median, axis, values)
+            return _wrap_results(np.apply_along_axis(get_median, axis, values), dtype)
 
         # must return the correct shape, but median is not defined for the
         # empty set so return nans of shape "everything but the passed axis"
@@ -311,10 +306,10 @@ def nanmedian(values, axis=None, skipna=True):
         dims = np.arange(values.ndim)
         ret = np.empty(shp[dims != axis])
         ret.fill(np.nan)
-        return ret
+        return _wrap_results(ret, dtype)
 
     # otherwise return a scalar value
-    return _wrap_results(get_median(values), dtype) if notempty else np.nan
+    return _wrap_results(get_median(values) if notempty else np.nan, dtype)
 
 
 def _get_counts_nanvar(mask, axis, ddof):
@@ -335,13 +330,11 @@ def _get_counts_nanvar(mask, axis, ddof):
     return count, d
 
 
-@disallow('M8')
-@bottleneck_switch(ddof=1)
-def nanvar(values, axis=None, skipna=True, ddof=1):
-    if not _is_floating_dtype(values):
-        values = values.astype('f8')
-
+def _nanvar(values, axis=None, skipna=True, ddof=1):
+    # private nanvar calculator
     mask = isnull(values)
+    if is_any_int_dtype(values):
+        values = values.astype('f8')
 
     count, d = _get_counts_nanvar(mask, axis, ddof)
 
@@ -353,13 +346,30 @@ def nanvar(values, axis=None, skipna=True, ddof=1):
     XX = _ensure_numeric((values ** 2).sum(axis))
     return np.fabs((XX - X ** 2 / count) / d)
 
+@disallow('M8')
+@bottleneck_switch(ddof=1)
+def nanstd(values, axis=None, skipna=True, ddof=1):
 
+    result = np.sqrt(_nanvar(values, axis=axis, skipna=skipna, ddof=ddof))
+    return _wrap_results(result, values.dtype)
+
+@disallow('M8','m8')
+@bottleneck_switch(ddof=1)
+def nanvar(values, axis=None, skipna=True, ddof=1):
+
+    # we are going to allow timedelta64[ns] here
+    # but NOT going to coerce them to the Timedelta type
+    # as this could cause overflow
+    # so var cannot be computed (but std can!)
+    return _nanvar(values, axis=axis, skipna=skipna, ddof=ddof)
+
+@disallow('M8','m8')
 def nansem(values, axis=None, skipna=True, ddof=1):
     var = nanvar(values, axis, skipna, ddof=ddof)
 
-    if not _is_floating_dtype(values):
-        values = values.astype('f8')
     mask = isnull(values)
+    if not is_floating_dtype(values):
+        values = values.astype('f8')
     count, _ = _get_counts_nanvar(mask, axis, ddof)
 
     return np.sqrt(var)/np.sqrt(count)
@@ -448,12 +458,13 @@ def nanargmin(values, axis=None, skipna=True):
     return result
 
 
-@disallow('M8')
+@disallow('M8','m8')
 def nanskew(values, axis=None, skipna=True):
-    if not _is_floating_dtype(values):
-        values = values.astype('f8')
 
     mask = isnull(values)
+    if not is_floating_dtype(values):
+        values = values.astype('f8')
+
     count = _get_counts(mask, axis)
 
     if skipna:
@@ -482,12 +493,13 @@ def nanskew(values, axis=None, skipna=True):
         return result
 
 
-@disallow('M8')
+@disallow('M8','m8')
 def nankurt(values, axis=None, skipna=True):
-    if not _is_floating_dtype(values):
-        values = values.astype('f8')
 
     mask = isnull(values)
+    if not is_floating_dtype(values):
+        values = values.astype('f8')
+
     count = _get_counts(mask, axis)
 
     if skipna:
@@ -500,26 +512,29 @@ def nankurt(values, axis=None, skipna=True):
     D = (values ** 4).sum(axis) / count - A ** 4 - 6 * B * A * A - 4 * C * A
 
     B = _zero_out_fperr(B)
-    C = _zero_out_fperr(C)
     D = _zero_out_fperr(D)
 
+    if not isinstance(B, np.ndarray):
+        # if B is a scalar, check these corner cases first before doing division
+        if count < 4:
+            return np.nan
+        if B == 0:
+            return 0
+
     result = (((count * count - 1.) * D / (B * B) - 3 * ((count - 1.) ** 2)) /
               ((count - 2.) * (count - 3.)))
+
     if isinstance(result, np.ndarray):
         result = np.where(B == 0, 0, result)
         result[count < 4] = np.nan
-        return result
-    else:
-        result = 0 if B == 0 else result
-        if count < 4:
-            return np.nan
-        return result
+
+    return result
 
 
-@disallow('M8')
+@disallow('M8','m8')
 def nanprod(values, axis=None, skipna=True):
     mask = isnull(values)
-    if skipna and not _is_any_int_dtype(values):
+    if skipna and not is_any_int_dtype(values):
         values = values.copy()
         values[mask] = 1
     result = values.prod(axis)
@@ -580,7 +595,7 @@ def _zero_out_fperr(arg):
         return 0 if np.abs(arg) < 1e-14 else arg
 
 
-@disallow('M8')
+@disallow('M8','m8')
 def nancorr(a, b, method='pearson', min_periods=None):
     """
     a, b: ndarrays
@@ -627,7 +642,7 @@ def get_corr_func(method):
     return _cor_methods[method]
 
 
-@disallow('M8')
+@disallow('M8','m8')
 def nancov(a, b, min_periods=None):
     if len(a) != len(b):
         raise AssertionError('Operands to nancov must have same size')
@@ -709,6 +724,10 @@ def unique1d(values):
         table = _hash.Int64HashTable(len(values))
         uniques = table.unique(_ensure_int64(values))
         uniques = uniques.view('M8[ns]')
+    elif np.issubdtype(values.dtype, np.timedelta64):
+        table = _hash.Int64HashTable(len(values))
+        uniques = table.unique(_ensure_int64(values))
+        uniques = uniques.view('m8[ns]')
     elif np.issubdtype(values.dtype, np.integer):
         table = _hash.Int64HashTable(len(values))
         uniques = table.unique(_ensure_int64(values))
