@@ -3,8 +3,13 @@
 import numpy as np
 from pandas.core.base import PandasDelegate
 from pandas.core import common as com
-from pandas import Series, DatetimeIndex, PeriodIndex, TimedeltaIndex
+from pandas.tseries.index import DatetimeIndex
+from pandas.tseries.period import PeriodIndex
+from pandas.tseries.tdi import TimedeltaIndex
 from pandas import lib, tslib
+from pandas.core.common import (_NS_DTYPE, _TD_DTYPE, is_period_arraylike,
+                                is_datetime_arraylike, is_integer_dtype, is_list_like,
+                                get_dtype_kinds)
 
 def is_datetimelike(data):
     """ return a boolean if we can be successfully converted to a datetimelike """
@@ -32,6 +37,7 @@ def maybe_to_datetimelike(data, copy=False):
     DelegatedClass
 
     """
+    from pandas import Series
 
     if not isinstance(data, Series):
         raise TypeError("cannot convert an object of type {0} to a datetimelike index".format(type(data)))
@@ -42,9 +48,9 @@ def maybe_to_datetimelike(data, copy=False):
     elif issubclass(data.dtype.type, np.timedelta64):
         return TimedeltaProperties(TimedeltaIndex(data, copy=copy, freq='infer'), index)
     else:
-        if com.is_period_arraylike(data):
+        if is_period_arraylike(data):
             return PeriodProperties(PeriodIndex(data, copy=copy), index)
-        if com.is_datetime_arraylike(data):
+        if is_datetime_arraylike(data):
             return DatetimeProperties(DatetimeIndex(data, copy=copy, freq='infer'), index)
 
     raise TypeError("cannot convert an object of type {0} to a datetimelike index".format(type(data)))
@@ -56,13 +62,15 @@ class Properties(PandasDelegate):
         self.index = index
 
     def _delegate_property_get(self, name):
+        from pandas import Series
+
         result = getattr(self.values,name)
 
         # maybe need to upcast (ints)
         if isinstance(result, np.ndarray):
-            if com.is_integer_dtype(result):
+            if is_integer_dtype(result):
                 result = result.astype('int64')
-        elif not com.is_list_like(result):
+        elif not is_list_like(result):
             return result
 
         # return the result as a Series, which is by definition a copy
@@ -79,6 +87,8 @@ class Properties(PandasDelegate):
                          "supported. Change values on the original.")
 
     def _delegate_method(self, name, *args, **kwargs):
+        from pandas import Series
+
         method = getattr(self.values, name)
         result = method(*args, **kwargs)
 
@@ -115,7 +125,7 @@ DatetimeProperties._add_delegate_accessors(delegate=DatetimeIndex,
                                            accessors=DatetimeIndex._datetimelike_ops,
                                            typ='property')
 DatetimeProperties._add_delegate_accessors(delegate=DatetimeIndex,
-                                           accessors=["to_period","tz_localize","tz_convert"],
+                                           accessors=["to_period","tz_localize","tz_convert","normalize"],
                                            typ='method')
 
 class TimedeltaProperties(Properties):
@@ -136,7 +146,16 @@ class TimedeltaProperties(Properties):
 
     @property
     def components(self):
-        return self.values.components
+        """
+        Return a dataframe of the components (days, hours, minutes,
+        seconds, milliseconds, microseconds, nanoseconds) of the Timedeltas.
+
+        Returns
+        -------
+        a DataFrame
+
+        """
+        return self.values.components.set_index(self.index)
 
 TimedeltaProperties._add_delegate_accessors(delegate=TimedeltaIndex,
                                             accessors=TimedeltaIndex._datetimelike_ops,
@@ -162,3 +181,58 @@ class PeriodProperties(Properties):
 PeriodProperties._add_delegate_accessors(delegate=PeriodIndex,
                                          accessors=PeriodIndex._datetimelike_ops,
                                          typ='property')
+
+
+class CombinedDatetimelikeProperties(DatetimeProperties, TimedeltaProperties):
+    # This class is never instantiated, and exists solely for the benefit of
+    # the Series.dt class property. For Series objects, .dt will always be one
+    # of the more specific classes above.
+    __doc__ = DatetimeProperties.__doc__
+
+
+def _concat_compat(to_concat, axis=0):
+    """
+    provide concatenation of an datetimelike array of arrays each of which is a single
+    M8[ns], or m8[ns] dtype
+
+    Parameters
+    ----------
+    to_concat : array of arrays
+    axis : axis to provide concatenation
+
+    Returns
+    -------
+    a single array, preserving the combined dtypes
+    """
+
+    def convert_to_pydatetime(x, axis):
+        # coerce to an object dtype
+        if x.dtype == _NS_DTYPE:
+            shape = x.shape
+            x = tslib.ints_to_pydatetime(x.view(np.int64).ravel())
+            x = x.reshape(shape)
+        elif x.dtype == _TD_DTYPE:
+            shape = x.shape
+            x = tslib.ints_to_pytimedelta(x.view(np.int64).ravel())
+            x = x.reshape(shape)
+        return x
+
+    typs = get_dtype_kinds(to_concat)
+
+    # single dtype
+    if len(typs) == 1:
+
+        if not len(typs-set(['datetime'])):
+            new_values = np.concatenate([x.view(np.int64) for x in to_concat],
+                                        axis=axis)
+            return new_values.view(_NS_DTYPE)
+
+        elif not len(typs-set(['timedelta'])):
+            new_values = np.concatenate([x.view(np.int64) for x in to_concat],
+                                        axis=axis)
+            return new_values.view(_TD_DTYPE)
+
+    # need to coerce to object
+    to_concat = [convert_to_pydatetime(x, axis) for x in to_concat]
+
+    return np.concatenate(to_concat,axis=axis)
