@@ -12,9 +12,18 @@ from pandas.tseries.base import DatetimeIndexOpsMixin
 from pandas.tseries.tools import parse_time_string
 import pandas.tseries.offsets as offsets
 
+from pandas._period import Period
+import pandas._period as period
+from pandas._period import (
+    get_period_field_arr,
+    _validate_end_alias,
+    _quarter_to_myear,
+)
+
 import pandas.core.common as com
 from pandas.core.common import (isnull, _INT64_DTYPE, _maybe_box,
-                                _values_from_object, ABCSeries)
+                                _values_from_object, ABCSeries,
+                                is_integer, is_float)
 from pandas import compat
 from pandas.lib import Timestamp, Timedelta
 import pandas.lib as lib
@@ -23,486 +32,19 @@ import pandas.algos as _algos
 from pandas.compat import zip, u
 
 
-#---------------
-# Period logic
-
-def _period_field_accessor(name, alias):
-    def f(self):
-        base, mult = _gfc(self.freq)
-        return tslib.get_period_field(alias, self.ordinal, base)
-    f.__name__ = name
-    return property(f)
-
-
 def _field_accessor(name, alias, docstring=None):
     def f(self):
         base, mult = _gfc(self.freq)
-        return tslib.get_period_field_arr(alias, self.values, base)
+        return get_period_field_arr(alias, self.values, base)
     f.__name__ = name
     f.__doc__ = docstring
     return property(f)
 
-class Period(PandasObject):
-    """
-    Represents an period of time
-
-    Parameters
-    ----------
-    value : Period or compat.string_types, default None
-        The time period represented (e.g., '4Q2005')
-    freq : str, default None
-        e.g., 'B' for businessday. Must be a singular rule-code (e.g. 5T is not
-        allowed).
-    year : int, default None
-    month : int, default 1
-    quarter : int, default None
-    day : int, default 1
-    hour : int, default 0
-    minute : int, default 0
-    second : int, default 0
-    """
-    __slots__ = ['freq', 'ordinal']
-    _comparables = ['name','freqstr']
-    _typ = 'period'
-
-    @classmethod
-    def _from_ordinal(cls, ordinal, freq):
-        """ fast creation from an ordinal and freq that are already validated! """
-        self = object.__new__(cls)
-        self.ordinal = ordinal
-        self.freq = freq
-        return self
-
-    def __init__(self, value=None, freq=None, ordinal=None,
-                 year=None, month=1, quarter=None, day=1,
-                 hour=0, minute=0, second=0):
-
-        # freq points to a tuple (base, mult);  base is one of the defined
-        # periods such as A, Q, etc. Every five minutes would be, e.g.,
-        # ('T', 5) but may be passed in as a string like '5T'
-
-        self.freq = None
-
-        # ordinal is the period offset from the gregorian proleptic epoch
-        self.ordinal = None
-
-        if ordinal is not None and value is not None:
-            raise ValueError(("Only value or ordinal but not both should be "
-                              "given but not both"))
-        elif ordinal is not None:
-            if not com.is_integer(ordinal):
-                raise ValueError("Ordinal must be an integer")
-            if freq is None:
-                raise ValueError('Must supply freq for ordinal value')
-            self.ordinal = ordinal
-
-        elif value is None:
-            if freq is None:
-                raise ValueError("If value is None, freq cannot be None")
-
-            self.ordinal = _ordinal_from_fields(year, month, quarter, day,
-                                                hour, minute, second, freq)
-
-        elif isinstance(value, Period):
-            other = value
-            if freq is None or _gfc(freq) == _gfc(other.freq):
-                self.ordinal = other.ordinal
-                freq = other.freq
-            else:
-                converted = other.asfreq(freq)
-                self.ordinal = converted.ordinal
-
-        elif com._is_null_datelike_scalar(value) or value in tslib._nat_strings:
-            self.ordinal = tslib.iNaT
-            if freq is None:
-                raise ValueError("If value is NaT, freq cannot be None "
-                                 "because it cannot be inferred")
-
-        elif isinstance(value, compat.string_types) or com.is_integer(value):
-            if com.is_integer(value):
-                value = str(value)
-            value = value.upper()
-
-            dt, _, reso = parse_time_string(value, freq)
-            if freq is None:
-                try:
-                    freq = frequencies.Resolution.get_freq(reso)
-                except KeyError:
-                    raise ValueError("Invalid frequency or could not infer: %s" % reso)
-
-        elif isinstance(value, datetime):
-            dt = value
-            if freq is None:
-                raise ValueError('Must supply freq for datetime value')
-        elif isinstance(value, date):
-            dt = datetime(year=value.year, month=value.month, day=value.day)
-            if freq is None:
-                raise ValueError('Must supply freq for datetime value')
-        else:
-            msg = "Value must be Period, string, integer, or datetime"
-            raise ValueError(msg)
-
-        base, mult = _gfc(freq)
-        if mult != 1:
-            # TODO: Better error message - this is slightly confusing
-            raise ValueError('Only mult == 1 supported')
-
-        if self.ordinal is None:
-            self.ordinal = tslib.period_ordinal(dt.year, dt.month, dt.day,
-                                                dt.hour, dt.minute, dt.second, dt.microsecond, 0,
-                                                base)
-
-        self.freq = frequencies._get_freq_str(base)
-
-    def __eq__(self, other):
-        if isinstance(other, Period):
-            if other.freq != self.freq:
-                raise ValueError("Cannot compare non-conforming periods")
-            if self.ordinal == tslib.iNaT or other.ordinal == tslib.iNaT:
-                return False
-            return (self.ordinal == other.ordinal
-                    and _gfc(self.freq) == _gfc(other.freq))
-        return NotImplemented
-
-    def __ne__(self, other):
-        return not self == other
-
-    def __hash__(self):
-        return hash((self.ordinal, self.freq))
-
-    def _add_delta(self, other):
-        if isinstance(other, (timedelta, np.timedelta64, offsets.Tick, Timedelta)):
-            offset = frequencies.to_offset(self.freq)
-            if isinstance(offset, offsets.Tick):
-                nanos = tslib._delta_to_nanoseconds(other)
-                offset_nanos = tslib._delta_to_nanoseconds(offset)
-
-                if nanos % offset_nanos == 0:
-                    if self.ordinal == tslib.iNaT:
-                        ordinal = self.ordinal
-                    else:
-                        ordinal = self.ordinal + (nanos // offset_nanos)
-                    return Period(ordinal=ordinal, freq=self.freq)
-        elif isinstance(other, offsets.DateOffset):
-            freqstr = frequencies.get_standard_freq(other)
-            base = frequencies.get_base_alias(freqstr)
-
-            if base == self.freq:
-                if self.ordinal == tslib.iNaT:
-                    ordinal = self.ordinal
-                else:
-                    ordinal = self.ordinal + other.n
-                return Period(ordinal=ordinal, freq=self.freq)
-
-        raise ValueError("Input has different freq from Period(freq={0})".format(self.freq))
-
-    def __add__(self, other):
-        if isinstance(other, (timedelta, np.timedelta64,
-                              offsets.Tick, offsets.DateOffset, Timedelta)):
-            return self._add_delta(other)
-        elif com.is_integer(other):
-            if self.ordinal == tslib.iNaT:
-                ordinal = self.ordinal
-            else:
-                ordinal = self.ordinal + other
-            return Period(ordinal=ordinal, freq=self.freq)
-        else:  # pragma: no cover
-            return NotImplemented
-
-    def __sub__(self, other):
-        if isinstance(other, (timedelta, np.timedelta64,
-                              offsets.Tick, offsets.DateOffset, Timedelta)):
-            neg_other = -other
-            return self + neg_other
-        elif com.is_integer(other):
-            if self.ordinal == tslib.iNaT:
-                ordinal = self.ordinal
-            else:
-                ordinal = self.ordinal - other
-            return Period(ordinal=ordinal, freq=self.freq)
-        elif isinstance(other, Period):
-            if other.freq != self.freq:
-                raise ValueError("Cannot do arithmetic with "
-                                 "non-conforming periods")
-            if self.ordinal == tslib.iNaT or other.ordinal == tslib.iNaT:
-                return Period(ordinal=tslib.iNaT, freq=self.freq)
-            return self.ordinal - other.ordinal
-        else:  # pragma: no cover
-            return NotImplemented
-
-    def _comp_method(func, name):
-        def f(self, other):
-            if isinstance(other, Period):
-                if other.freq != self.freq:
-                    raise ValueError("Cannot compare non-conforming periods")
-                if self.ordinal == tslib.iNaT or other.ordinal == tslib.iNaT:
-                    return False
-                return func(self.ordinal, other.ordinal)
-            else:
-                raise TypeError(other)
-
-        f.__name__ = name
-        return f
-
-    __lt__ = _comp_method(operator.lt, '__lt__')
-    __le__ = _comp_method(operator.le, '__le__')
-    __gt__ = _comp_method(operator.gt, '__gt__')
-    __ge__ = _comp_method(operator.ge, '__ge__')
-
-    def asfreq(self, freq, how='E'):
-        """
-        Convert Period to desired frequency, either at the start or end of the
-        interval
-
-        Parameters
-        ----------
-        freq : string
-        how : {'E', 'S', 'end', 'start'}, default 'end'
-            Start or end of the timespan
-
-        Returns
-        -------
-        resampled : Period
-        """
-        how = _validate_end_alias(how)
-        base1, mult1 = _gfc(self.freq)
-        base2, mult2 = _gfc(freq)
-
-        if mult2 != 1:
-            raise ValueError('Only mult == 1 supported')
-
-        end = how == 'E'
-        new_ordinal = tslib.period_asfreq(self.ordinal, base1, base2, end)
-
-        return Period(ordinal=new_ordinal, freq=base2)
-
-    @property
-    def start_time(self):
-        return self.to_timestamp(how='S')
-
-    @property
-    def end_time(self):
-        if self.ordinal == tslib.iNaT:
-            ordinal = self.ordinal
-        else:
-            ordinal = (self + 1).start_time.value - 1
-        return Timestamp(ordinal)
-
-    def to_timestamp(self, freq=None, how='start', tz=None):
-        """
-        Return the Timestamp representation of the Period at the target
-        frequency at the specified end (how) of the Period
-
-        Parameters
-        ----------
-        freq : string or DateOffset, default is 'D' if self.freq is week or
-               longer and 'S' otherwise
-            Target frequency
-        how: str, default 'S' (start)
-            'S', 'E'. Can be aliased as case insensitive
-            'Start', 'Finish', 'Begin', 'End'
-
-        Returns
-        -------
-        Timestamp
-        """
-        how = _validate_end_alias(how)
-
-        if freq is None:
-            base, mult = _gfc(self.freq)
-            freq = frequencies.get_to_timestamp_base(base)
-
-        base, mult = _gfc(freq)
-        val = self.asfreq(freq, how)
-
-        dt64 = tslib.period_ordinal_to_dt64(val.ordinal, base)
-        return Timestamp(dt64, tz=tz)
-
-    year = _period_field_accessor('year', 0)
-    month = _period_field_accessor('month', 3)
-    day = _period_field_accessor('day', 4)
-    hour = _period_field_accessor('hour', 5)
-    minute = _period_field_accessor('minute', 6)
-    second = _period_field_accessor('second', 7)
-    weekofyear = _period_field_accessor('week', 8)
-    week = weekofyear
-    dayofweek = _period_field_accessor('dayofweek', 10)
-    weekday = dayofweek
-    dayofyear = _period_field_accessor('dayofyear', 9)
-    quarter = _period_field_accessor('quarter', 2)
-    qyear = _period_field_accessor('qyear', 1)
-
-    @classmethod
-    def now(cls, freq=None):
-        return Period(datetime.now(), freq=freq)
-
-    def __repr__(self):
-        base, mult = _gfc(self.freq)
-        formatted = tslib.period_format(self.ordinal, base)
-        freqstr = frequencies._reverse_period_code_map[base]
-
-        if not compat.PY3:
-            encoding = com.get_option("display.encoding")
-            formatted = formatted.encode(encoding)
-
-        return "Period('%s', '%s')" % (formatted, freqstr)
-
-    def __unicode__(self):
-        """
-        Return a string representation for a particular DataFrame
-
-        Invoked by unicode(df) in py2 only. Yields a Unicode String in both
-        py2/py3.
-        """
-        base, mult = _gfc(self.freq)
-        formatted = tslib.period_format(self.ordinal, base)
-        value = ("%s" % formatted)
-        return value
-
-    def strftime(self, fmt):
-        """
-        Returns the string representation of the :class:`Period`, depending
-        on the selected :keyword:`format`. :keyword:`format` must be a string
-        containing one or several directives.  The method recognizes the same
-        directives as the :func:`time.strftime` function of the standard Python
-        distribution, as well as the specific additional directives ``%f``,
-        ``%F``, ``%q``. (formatting & docs originally from scikits.timeries)
-
-        +-----------+--------------------------------+-------+
-        | Directive | Meaning                        | Notes |
-        +===========+================================+=======+
-        | ``%a``    | Locale's abbreviated weekday   |       |
-        |           | name.                          |       |
-        +-----------+--------------------------------+-------+
-        | ``%A``    | Locale's full weekday name.    |       |
-        +-----------+--------------------------------+-------+
-        | ``%b``    | Locale's abbreviated month     |       |
-        |           | name.                          |       |
-        +-----------+--------------------------------+-------+
-        | ``%B``    | Locale's full month name.      |       |
-        +-----------+--------------------------------+-------+
-        | ``%c``    | Locale's appropriate date and  |       |
-        |           | time representation.           |       |
-        +-----------+--------------------------------+-------+
-        | ``%d``    | Day of the month as a decimal  |       |
-        |           | number [01,31].                |       |
-        +-----------+--------------------------------+-------+
-        | ``%f``    | 'Fiscal' year without a        | \(1)  |
-        |           | century  as a decimal number   |       |
-        |           | [00,99]                        |       |
-        +-----------+--------------------------------+-------+
-        | ``%F``    | 'Fiscal' year with a century   | \(2)  |
-        |           | as a decimal number            |       |
-        +-----------+--------------------------------+-------+
-        | ``%H``    | Hour (24-hour clock) as a      |       |
-        |           | decimal number [00,23].        |       |
-        +-----------+--------------------------------+-------+
-        | ``%I``    | Hour (12-hour clock) as a      |       |
-        |           | decimal number [01,12].        |       |
-        +-----------+--------------------------------+-------+
-        | ``%j``    | Day of the year as a decimal   |       |
-        |           | number [001,366].              |       |
-        +-----------+--------------------------------+-------+
-        | ``%m``    | Month as a decimal number      |       |
-        |           | [01,12].                       |       |
-        +-----------+--------------------------------+-------+
-        | ``%M``    | Minute as a decimal number     |       |
-        |           | [00,59].                       |       |
-        +-----------+--------------------------------+-------+
-        | ``%p``    | Locale's equivalent of either  | \(3)  |
-        |           | AM or PM.                      |       |
-        +-----------+--------------------------------+-------+
-        | ``%q``    | Quarter as a decimal number    |       |
-        |           | [01,04]                        |       |
-        +-----------+--------------------------------+-------+
-        | ``%S``    | Second as a decimal number     | \(4)  |
-        |           | [00,61].                       |       |
-        +-----------+--------------------------------+-------+
-        | ``%U``    | Week number of the year        | \(5)  |
-        |           | (Sunday as the first day of    |       |
-        |           | the week) as a decimal number  |       |
-        |           | [00,53].  All days in a new    |       |
-        |           | year preceding the first       |       |
-        |           | Sunday are considered to be in |       |
-        |           | week 0.                        |       |
-        +-----------+--------------------------------+-------+
-        | ``%w``    | Weekday as a decimal number    |       |
-        |           | [0(Sunday),6].                 |       |
-        +-----------+--------------------------------+-------+
-        | ``%W``    | Week number of the year        | \(5)  |
-        |           | (Monday as the first day of    |       |
-        |           | the week) as a decimal number  |       |
-        |           | [00,53].  All days in a new    |       |
-        |           | year preceding the first       |       |
-        |           | Monday are considered to be in |       |
-        |           | week 0.                        |       |
-        +-----------+--------------------------------+-------+
-        | ``%x``    | Locale's appropriate date      |       |
-        |           | representation.                |       |
-        +-----------+--------------------------------+-------+
-        | ``%X``    | Locale's appropriate time      |       |
-        |           | representation.                |       |
-        +-----------+--------------------------------+-------+
-        | ``%y``    | Year without century as a      |       |
-        |           | decimal number [00,99].        |       |
-        +-----------+--------------------------------+-------+
-        | ``%Y``    | Year with century as a decimal |       |
-        |           | number.                        |       |
-        +-----------+--------------------------------+-------+
-        | ``%Z``    | Time zone name (no characters  |       |
-        |           | if no time zone exists).       |       |
-        +-----------+--------------------------------+-------+
-        | ``%%``    | A literal ``'%'`` character.   |       |
-        +-----------+--------------------------------+-------+
-
-        .. note::
-
-            (1)
-                The ``%f`` directive is the same as ``%y`` if the frequency is
-                not quarterly.
-                Otherwise, it corresponds to the 'fiscal' year, as defined by
-                the :attr:`qyear` attribute.
-
-            (2)
-                The ``%F`` directive is the same as ``%Y`` if the frequency is
-                not quarterly.
-                Otherwise, it corresponds to the 'fiscal' year, as defined by
-                the :attr:`qyear` attribute.
-
-            (3)
-                The ``%p`` directive only affects the output hour field
-                if the ``%I`` directive is used to parse the hour.
-
-            (4)
-                The range really is ``0`` to ``61``; this accounts for leap
-                seconds and the (very rare) double leap seconds.
-
-            (5)
-                The ``%U`` and ``%W`` directives are only used in calculations
-                when the day of the week and the year are specified.
-
-        .. rubric::  Examples
-
-            >>> a = Period(freq='Q@JUL', year=2006, quarter=1)
-            >>> a.strftime('%F-Q%q')
-            '2006-Q1'
-            >>> # Output the last month in the quarter of this date
-            >>> a.strftime('%b-%Y')
-            'Oct-2005'
-            >>>
-            >>> a = Period(freq='D', year=2001, month=1, day=1)
-            >>> a.strftime('%d-%b-%Y')
-            '01-Jan-2006'
-            >>> a.strftime('%b. %d, %Y was a %A')
-            'Jan. 01, 2001 was a Monday'
-        """
-        base, mult = _gfc(self.freq)
-        return tslib.period_format(self.ordinal, base, fmt)
 
 def _get_ordinals(data, freq):
     f = lambda x: Period(x, freq=freq).ordinal
     if isinstance(data[0], Period):
-        return tslib.extract_ordinals(data, freq)
+        return period.extract_ordinals(data, freq)
     else:
         return lib.map_infer(data, f)
 
@@ -512,7 +54,7 @@ def dt64arr_to_periodarr(data, freq, tz):
         raise ValueError('Wrong dtype: %s' % data.dtype)
 
     base, mult = _gfc(freq)
-    return tslib.dt64arr_to_periodarr(data.view('i8'), base, tz)
+    return period.dt64arr_to_periodarr(data.view('i8'), base, tz)
 
 # --- Period index sketch
 
@@ -608,7 +150,7 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
     _typ = 'periodindex'
     _attributes = ['name','freq']
     _datetimelike_ops = ['year','month','day','hour','minute','second',
-                         'weekofyear','week','dayofweek','weekday','dayofyear','quarter', 'qyear', 'freq']
+                         'weekofyear','week','dayofweek','weekday','dayofyear','quarter', 'qyear', 'freq', 'days_in_month', 'daysinmonth']
     _is_numeric_dtype = False
     freq = None
 
@@ -625,9 +167,9 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         freq = frequencies.get_standard_freq(freq)
 
         if periods is not None:
-            if com.is_float(periods):
+            if is_float(periods):
                 periods = int(periods)
-            elif not com.is_integer(periods):
+            elif not is_integer(periods):
                 raise ValueError('Periods must be a number, got %s' %
                                  str(periods))
 
@@ -697,7 +239,7 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
                 else:
                     base1, _ = _gfc(data.freq)
                     base2, _ = _gfc(freq)
-                    data = tslib.period_asfreq_arr(data.values, base1,
+                    data = period.period_asfreq_arr(data.values, base1,
                                                    base2, 1)
             else:
                 if freq is None and len(data) > 0:
@@ -751,6 +293,10 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         """ return an array repr of this object, potentially casting to object """
         return self.asobject.values
 
+    @property
+    def _formatter_func(self):
+        return lambda x: "'%s'" % x
+
     def asof_locs(self, where, mask):
         """
         where : array of timestamps
@@ -813,6 +359,44 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         return self.freq
 
     def asfreq(self, freq=None, how='E'):
+        """
+        Convert the PeriodIndex to the specified frequency `freq`.
+
+        Parameters
+        ----------
+
+        freq : str
+            a frequency
+        how : str {'E', 'S'}
+            'E', 'END', or 'FINISH' for end,
+            'S', 'START', or 'BEGIN' for start.
+            Whether the elements should be aligned to the end
+            or start within pa period. January 31st ('END') vs.
+            Janury 1st ('START') for example.
+
+        Returns
+        -------
+
+        new : PeriodIndex with the new frequency
+
+        Examples
+        --------
+        >>> pidx = pd.period_range('2010-01-01', '2015-01-01', freq='A')
+        >>> pidx
+        <class 'pandas.tseries.period.PeriodIndex'>
+        [2010, ..., 2015]
+        Length: 6, Freq: A-DEC
+
+        >>> pidx.asfreq('M')
+        <class 'pandas.tseries.period.PeriodIndex'>
+        [2010-12, ..., 2015-12]
+        Length: 6, Freq: M
+
+        >>> pidx.asfreq('M', how='S')
+        <class 'pandas.tseries.period.PeriodIndex'>
+        [2010-01, ..., 2015-01]
+        Length: 6, Freq: M
+        """
         how = _validate_end_alias(how)
 
         freq = frequencies.get_standard_freq(freq)
@@ -824,7 +408,7 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
             raise ValueError('Only mult == 1 supported')
 
         end = how == 'E'
-        new_data = tslib.period_asfreq_arr(self.values, base1, base2, end)
+        new_data = period.period_asfreq_arr(self.values, base1, base2, end)
         return self._simple_new(new_data, self.name, freq=freq)
 
     def to_datetime(self, dayfirst=False):
@@ -843,6 +427,8 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
     dayofyear = day_of_year = _field_accessor('dayofyear', 9, "The ordinal day of the year")
     quarter = _field_accessor('quarter', 2, "The quarter of the date")
     qyear = _field_accessor('qyear', 1)
+    days_in_month = _field_accessor('days_in_month', 11, "The number of days in the month")
+    daysinmonth = days_in_month
 
     def _get_object_array(self):
         freq = self.freq
@@ -892,7 +478,7 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         base, mult = _gfc(freq)
         new_data = self.asfreq(freq, how)
 
-        new_data = tslib.periodarr_to_dt64arr(new_data.values, base)
+        new_data = period.periodarr_to_dt64arr(new_data.values, base)
         return DatetimeIndex(new_data, freq='infer', name=self.name)
 
     def _add_delta(self, other):
@@ -975,7 +561,13 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
             key = Period(key, self.freq).ordinal
             return _maybe_box(self, self._engine.get_value(s, key), series, key)
 
-    def get_loc(self, key):
+    def get_indexer(self, target, method=None, limit=None):
+        if hasattr(target, 'freq') and target.freq != self.freq:
+            raise ValueError('target and index have different freq: '
+                             '(%s, %s)' % (target.freq, self.freq))
+        return Index.get_indexer(self, target, method, limit)
+
+    def get_loc(self, key, method=None):
         """
         Get integer location for requested label
 
@@ -986,7 +578,7 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         try:
             return self._engine.get_loc(key)
         except KeyError:
-            if com.is_integer(key):
+            if is_integer(key):
                 raise
 
             try:
@@ -997,11 +589,11 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
 
             key = Period(key, self.freq)
             try:
-                return self._engine.get_loc(key.ordinal)
+                return Index.get_loc(self, key.ordinal, method=method)
             except KeyError:
                 raise KeyError(key)
 
-    def _maybe_cast_slice_bound(self, label, side):
+    def _maybe_cast_slice_bound(self, label, side, kind):
         """
         If label is a string or a datetime, cast it to Period.ordinal according to
         resolution.
@@ -1010,6 +602,7 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         ----------
         label : object
         side : {'left', 'right'}
+        kind : string / None
 
         Returns
         -------
@@ -1029,6 +622,8 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
                 return bounds[0 if side == 'left' else 1]
             except Exception:
                 raise KeyError(label)
+        elif is_integer(label) or is_float(label):
+            self._invalid_indexer('slice',label)
 
         return label
 
@@ -1113,7 +708,7 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
             val = getitem(key)
             return Period(ordinal=val, freq=self.freq)
         else:
-            if com._is_bool_indexer(key):
+            if com.is_bool_indexer(key):
                 key = np.asarray(key)
 
             result = getitem(key)
@@ -1134,7 +729,7 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
 
         imask = ~mask
         values[imask] = np.array([u('%s') % dt for dt in values[imask]])
-        return values.tolist()
+        return values
 
     def __array_finalize__(self, obj):
         if not self.ndim:  # pragma: no cover
@@ -1144,10 +739,6 @@ class PeriodIndex(DatetimeIndexOpsMixin, Int64Index):
         self.name = getattr(obj, 'name', None)
         self._reset_identity()
 
-    def _format_footer(self):
-        tagline = 'Length: %d, Freq: %s'
-        return tagline % (len(self), self.freqstr)
-
     def take(self, indices, axis=None):
         """
         Analogous to ndarray.take
@@ -1342,7 +933,7 @@ def _range_from_fields(year=None, month=None, quarter=None, day=None,
         year, quarter = _make_field_arrays(year, quarter)
         for y, q in zip(year, quarter):
             y, m = _quarter_to_myear(y, q, freq)
-            val = tslib.period_ordinal(y, m, 1, 1, 1, 1, 0, 0, base)
+            val = period.period_ordinal(y, m, 1, 1, 1, 1, 0, 0, base)
             ordinals.append(val)
     else:
         base, mult = _gfc(freq)
@@ -1351,7 +942,7 @@ def _range_from_fields(year=None, month=None, quarter=None, day=None,
 
         arrays = _make_field_arrays(year, month, day, hour, minute, second)
         for y, mth, d, h, mn, s in zip(*arrays):
-            ordinals.append(tslib.period_ordinal(y, mth, d, h, mn, s, 0, 0, base))
+            ordinals.append(period.period_ordinal(y, mth, d, h, mn, s, 0, 0, base))
 
     return np.array(ordinals, dtype=np.int64), freq
 
@@ -1371,41 +962,6 @@ def _make_field_arrays(*fields):
     return arrays
 
 
-def _ordinal_from_fields(year, month, quarter, day, hour, minute,
-                         second, freq):
-    base, mult = _gfc(freq)
-    if mult != 1:
-        raise ValueError('Only mult == 1 supported')
-
-    if quarter is not None:
-        year, month = _quarter_to_myear(year, quarter, freq)
-
-    return tslib.period_ordinal(year, month, day, hour, minute, second, 0, 0, base)
-
-
-def _quarter_to_myear(year, quarter, freq):
-    if quarter is not None:
-        if quarter <= 0 or quarter > 4:
-            raise ValueError('Quarter must be 1 <= q <= 4')
-
-        mnum = frequencies._month_numbers[frequencies._get_rule_month(freq)] + 1
-        month = (mnum + (quarter - 1) * 3) % 12 + 1
-        if month > mnum:
-            year -= 1
-
-    return year, month
-
-
-def _validate_end_alias(how):
-    how_dict = {'S': 'S', 'E': 'E',
-                'START': 'S', 'FINISH': 'E',
-                'BEGIN': 'S', 'END': 'E'}
-    how = how_dict.get(str(how).upper())
-    if how not in set(['S', 'E']):
-        raise ValueError('How must be one of S or E')
-    return how
-
-
 def pnow(freq=None):
     return Period(datetime.now(), freq=freq)
 
